@@ -3,7 +3,7 @@
    | PHP Version 5 / Imagick											  |
    +----------------------------------------------------------------------+
    | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar				  |
-   | Imagemagick (c) ImageMagick Studio LLC								  |
+   | ImageMagick (c) ImageMagick Studio LLC								  |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,	  |
    | that is bundled with this package in the file LICENSE, and is		  |
@@ -55,7 +55,7 @@ void php_imagick_cleanup_progress_callback(php_imagick_callback* progress_callba
 			efree(progress_callback->previous_callback);
 		}
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 		zval_ptr_dtor(&progress_callback->user_callback);
 #else
 		zval_ptr_dtor(&progress_callback->user_callback);
@@ -69,7 +69,7 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 	zend_fcall_info fci;
 	zend_fcall_info_cache fci_cache;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval zargs[2];
 	zval retval;
 #else
@@ -81,36 +81,40 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 	//IMAGICK_G(progress_callback) - this should be quicker
 	php_imagick_callback *callback = (php_imagick_callback*)userData;
 
-#if defined(ZEND_ENGINE_3) && defined(ZTS)
+	// This suppresses an 'unused parameter' warning.
+	// We could maybe use text.
+	(void)text;
+
+#if PHP_VERSION_ID >= 70000 && defined(ZTS)
 	if( NULL == tsrm_get_ls_cache() ) {
 		return MagickTrue;
 	}
 #endif
 
-#ifndef ZEND_ENGINE_3
+#if PHP_VERSION_ID < 70000
 	TSRMLS_FETCH_FROM_CTX(callback->thread_ctx);
 #endif
 	fci_cache = empty_fcall_info_cache;
 
+	fci = empty_fcall_info;
 	fci.size = sizeof(fci);
+
+#if PHP_VERSION_ID < 70100
 	fci.function_table = EG(function_table);
-#ifdef ZEND_ENGINE_3
-	fci.object = NULL;
+#endif
+#if PHP_VERSION_ID >= 70000
 	//fci.function_name = *callback->user_callback;
 	ZVAL_COPY_VALUE(&fci.function_name, &callback->user_callback);
 	fci.retval = &retval;
 #else
 	retval_ptr = NULL;
-	fci.object_ptr = NULL;
 	fci.function_name = callback->user_callback;
 	fci.retval_ptr_ptr = &retval_ptr;
 #endif
 	fci.param_count = 2;
 	fci.params = zargs;
-	fci.no_separation = 0;
-	fci.symbol_table = NULL;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZVAL_LONG(&zargs[0], offset);
     ZVAL_LONG(&zargs[1], span);
 #else
@@ -130,7 +134,7 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 		//Abort processing as user callback is no longer callable
 		return MagickFalse;
 	}
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	if (Z_TYPE(retval) == IS_FALSE) {
 		//User returned false - tell Imagick to abort processing.
 		return MagickFalse;
@@ -157,6 +161,10 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 */
 static inline double im_round_helper(double value) {
 	if (value >= 0.0) {
+		// Prevent zero width/height images
+		if (value < 1) {
+			return 1;
+		}
 		return floor(value + 0.5);
 	} else {
 		return ceil(value - 0.5);
@@ -277,7 +285,7 @@ double *php_imagick_zval_to_double_array(zval *param_array, im_long *num_element
 	double *double_array;
 	long i = 0;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval *pzvalue;
 #else
 	zval **ppzval;
@@ -291,7 +299,7 @@ double *php_imagick_zval_to_double_array(zval *param_array, im_long *num_element
 
 	double_array = ecalloc(*num_elements, sizeof(double));
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
 		ZVAL_DEREF(pzvalue);
 		double_array[i] = zval_get_double(pzvalue);
@@ -323,12 +331,12 @@ double *php_imagick_zval_to_double_array(zval *param_array, im_long *num_element
 	return double_array;
 }
 
-long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRMLS_DC)
+im_long *php_imagick_zval_to_long_array(zval *param_array, im_long *num_elements TSRMLS_DC)
 {
-	long *long_array;
-	long i = 0;
+	im_long *long_array;
+	im_long i = 0;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval *pzvalue;
 #else
 	zval **ppzval;
@@ -340,9 +348,9 @@ long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRML
 		return NULL;
 	}
 
-	long_array = ecalloc(*num_elements, sizeof(long));
+	long_array = ecalloc(*num_elements, sizeof(im_long));
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
 		ZVAL_DEREF(pzvalue);
 		long_array[i] = zval_get_long(pzvalue);
@@ -354,7 +362,7 @@ long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRML
 			zend_hash_move_forward(Z_ARRVAL_P(param_array)), i++)
 	{
 		zval tmp_zval, *tmp_pzval;
- 		long value = 0;
+ 		im_long value = 0;
 
 		if (Z_TYPE_PP(ppzval) == IS_DOUBLE) {
 			value = Z_LVAL_PP(ppzval);
@@ -375,12 +383,12 @@ long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRML
 	return long_array;
 }
 
-unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_elements TSRMLS_DC)
+unsigned char *php_imagick_zval_to_char_array(zval *param_array, im_long *num_elements TSRMLS_DC)
 {
 	unsigned char *char_array;
-	long i = 0;
+	im_long i = 0;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval *pzvalue;
 #else
 	zval **ppzval;
@@ -394,7 +402,7 @@ unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_eleme
 
 	char_array = ecalloc(*num_elements, sizeof(unsigned char));
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
 		ZVAL_DEREF(pzvalue);
 		char_array[i] = zval_get_long(pzvalue);
@@ -406,7 +414,7 @@ unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_eleme
 			zend_hash_move_forward(Z_ARRVAL_P(param_array)), i++)
 	{
 		zval tmp_zval, *tmp_pzval;
-		long value = 0;
+		im_long value = 0;
 		if (Z_TYPE_PP(ppzval) == IS_DOUBLE) {
 			value = Z_LVAL_PP(ppzval);
 		}
@@ -530,7 +538,7 @@ PointInfo *php_imagick_zval_to_pointinfo_array(zval *coordinate_array, int *num_
 	long elements, sub_elements, i;
 	HashTable *sub_array;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval *pzvalue;
 #else
 	HashTable *coords;
@@ -550,7 +558,7 @@ PointInfo *php_imagick_zval_to_pointinfo_array(zval *coordinate_array, int *num_
 	*num_elements = elements;
 	coordinates = emalloc(sizeof(PointInfo) * elements);
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(coordinate_array), pzvalue) {
 		zval *pz_x, *pz_y;
 		ZVAL_DEREF(pzvalue);
@@ -711,7 +719,7 @@ void s_convert_exception (char *description, const char *default_message, long s
 }
 
 /**
-	Convert image magick MagickWand exception to PHP exception
+	Convert ImageMagick MagickWand exception to PHP exception
 */
 void php_imagick_convert_imagick_exception (MagickWand *magick_wand, const char *default_message TSRMLS_DC)
 {
@@ -762,7 +770,7 @@ PixelWand *php_imagick_zval_to_pixelwand (zval *param, php_imagick_class_type_t
 	PixelWand *pixel_wand = NULL;
 	*allocated = 0;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZVAL_DEREF(param);
 #endif 
 	if (Z_TYPE_P (param) == IS_LONG || Z_TYPE_P (param) == IS_DOUBLE) {
@@ -792,7 +800,7 @@ PixelWand *php_imagick_zval_to_pixelwand (zval *param, php_imagick_class_type_t
 		break;
 
 		case IS_OBJECT:
-			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) {
+			if (instanceof_function(Z_OBJCE_P(param), php_imagickpixel_sc_entry TSRMLS_CC)) {
 				php_imagickpixel_object *intern = Z_IMAGICKPIXEL_P(param);
 				pixel_wand = intern->pixel_wand;
 			} else
@@ -810,7 +818,7 @@ PixelWand *php_imagick_zval_to_opacity (zval *param, php_imagick_class_type_t ca
 	PixelWand *pixel_wand = NULL;
 	*allocated = 0;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	ZVAL_DEREF(param);
 #endif
 	if (Z_TYPE_P (param) == IS_STRING) {
@@ -842,7 +850,7 @@ PixelWand *php_imagick_zval_to_opacity (zval *param, php_imagick_class_type_t ca
 		break;
 
 		case IS_OBJECT:
-			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) {
+			if (instanceof_function(Z_OBJCE_P(param), php_imagickpixel_sc_entry TSRMLS_CC)) {
 				php_imagickpixel_object *intern = Z_IMAGICKPIXEL_P(param);
 				pixel_wand = intern->pixel_wand;
 			} else
@@ -946,6 +954,16 @@ zend_bool php_imagick_ensure_not_empty (MagickWand *magick_wand)
 	return 1;
 }
 
+zend_bool php_imagickpixel_ensure_not_null(PixelWand *pixel_wand)
+{
+	if (pixel_wand == NULL) {
+		TSRMLS_FETCH ();
+		php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Can not process empty ImagickPixel object" TSRMLS_CC);
+		return 0;
+	}
+	return 1;
+}
+
 void php_imagick_initialize_constants(TSRMLS_D)
 {
 #define IMAGICK_REGISTER_CONST_LONG(const_name, value)\
@@ -1087,6 +1105,24 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_HARDMIX", HardMixCompositeOp);
 #endif
 
+#if (MagickLibVersion >= 0x700 && MagickLibVersion >= 0x708) || (MagickLibVersion < 0x700 && MagickLibVersion >= 0x69A)
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_STEREO", StereoCompositeOp);
+#endif
+
+#if MagickLibVersion >= 0x70A
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_FREEZE", FreezeCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_INTERPOLATE", InterpolateCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_NEGATE", NegateCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_REFLECT", ReflectCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SOFTBURN", SoftBurnCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SOFTDODGE", SoftDodgeCompositeOp);
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_STAMP", StampCompositeOp);
+#endif
+
+#if MagickLibVersion >= 0x70B
+    IMAGICK_REGISTER_CONST_LONG("COMPOSITE_RMSE", RMSECompositeOp);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_FRAME", FrameMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_UNFRAME", UnframeMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_CONCATENATE", ConcatenateMode);
@@ -1094,12 +1130,21 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("STYLE_ITALIC", ItalicStyle);
 	IMAGICK_REGISTER_CONST_LONG("STYLE_OBLIQUE", ObliqueStyle);
 	IMAGICK_REGISTER_CONST_LONG("STYLE_ANY", AnyStyle);
+#if (MagickLibVersion >= 0x700 && MagickLibVersion >= 0x709) || (MagickLibVersion < 0x700 && MagickLibVersion >= 0x69B)
+    IMAGICK_REGISTER_CONST_LONG("STYLE_BOLD", BoldStyle);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("FILTER_UNDEFINED", UndefinedFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_POINT", PointFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_BOX", BoxFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_TRIANGLE", TriangleFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_HERMITE", HermiteFilter);
+#if MagickLibVersion >= 0x701
+	IMAGICK_REGISTER_CONST_LONG("FILTER_HANNING", HannFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_HANN", HannFilter);
+#else
 	IMAGICK_REGISTER_CONST_LONG("FILTER_HANNING", HanningFilter);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("FILTER_HAMMING", HammingFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_BLACKMAN", BlackmanFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_GAUSSIAN", GaussianFilter);
@@ -1112,7 +1157,12 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("FILTER_SINC", SincFilter);
 #if MagickLibVersion >= 0x637
 	IMAGICK_REGISTER_CONST_LONG("FILTER_KAISER", KaiserFilter);
+#if MagickLibVersion >= 0x701
 	IMAGICK_REGISTER_CONST_LONG("FILTER_WELSH", WelshFilter);
+	IMAGICK_REGISTER_CONST_LONG("FILTER_WELCH", WelchFilter);
+#else
+	IMAGICK_REGISTER_CONST_LONG("FILTER_WELSH", WelshFilter);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("FILTER_PARZEN", ParzenFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_LAGRANGE", LagrangeFilter);
 	IMAGICK_REGISTER_CONST_LONG("FILTER_SENTINEL", SentinelFilter);
@@ -1138,28 +1188,52 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 #if MagickLibVersion >= 0x681
 	IMAGICK_REGISTER_CONST_LONG("FILTER_LANCZOSRADIUS", LanczosRadiusFilter);
+#endif
+#if MagickLibVersion >= 0x707
+	IMAGICK_REGISTER_CONST_LONG("FILTER_CUBIC_SPLINE", CubicSplineFilter);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_UNDEFINED", UndefinedType);
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_BILEVEL", BilevelType);
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALE", GrayscaleType);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEMATTE", GrayscaleMatteType);
+#else
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEALPHA", GrayscaleAlphaType);
+	//@TODO - this is only here for legacy support
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEMATTE", GrayscaleAlphaType);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTE",  PaletteType);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEMATTE", PaletteMatteType);
+#else
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEMATTE", PaletteAlphaType);
+	//@TODO - this is only here for legacy support
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEALPHA", PaletteAlphaType);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLOR", TrueColorType);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORMATTE", TrueColorMatteType);
+#else
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORALPHA", TrueColorAlphaType);
+	//@TODO - this is only here for legacy support
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORMATTE", TrueColorAlphaType);
+	
 #endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATION", ColorSeparationType);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONMATTE", ColorSeparationMatteType);
+#else
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONALPHA", ColorSeparationAlphaType);
+	//@TODO - this is only here for legacy support
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONMATTE", ColorSeparationAlphaType);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_OPTIMIZE", OptimizeType);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEBILEVELMATTE", PaletteBilevelMatteType);
+#else
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEBILEVELALPHA", PaletteBilevelAlphaType);
+	//@TODO - this is only here for legacy support
+	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEBILEVELMATTE", PaletteBilevelAlphaType);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_UNDEFINED", UndefinedResolution);
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERINCH", PixelsPerInchResolution);
@@ -1194,6 +1268,17 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_JBIG1", JBIG1Compression);
 	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_JBIG2", JBIG2Compression);
 #endif
+
+#if MagickLibVersion >= 0x709
+	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_ZSTD", ZstdCompression);
+	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_WEBP", WebPCompression);
+#endif
+
+#if MagickLibVersion >= 0x70C
+	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_DWAA", DWAACompression);
+	IMAGICK_REGISTER_CONST_LONG("COMPRESSION_DWAB", DWABCompression);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("PAINT_POINT", PointMethod);
 	IMAGICK_REGISTER_CONST_LONG("PAINT_REPLACE", ReplaceMethod);
 	IMAGICK_REGISTER_CONST_LONG("PAINT_FLOODFILL", FloodfillMethod);
@@ -1276,6 +1361,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_META", MetaChannel);              /* ???? */
 #endif
 
+#if MagickLibVersion >= 0x708
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_COMPOSITE_MASK", CompositeMaskChannel);              /* SVG mask */
+#endif
+
 #if MagickLibVersion >= 0x670
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_COMPOSITES", CompositeChannels);
 #endif
@@ -1286,11 +1375,16 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANABSOLUTEERROR", MeanAbsoluteErrorMetric);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANERRORPERPIXELMETRIC", MeanErrorPerPixelMetric);
+#else 
+	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANERRORPERPIXELMETRIC", MeanErrorPerPixelErrorMetric);
 #endif
+
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANSQUAREERROR", MeanSquaredErrorMetric);
 	IMAGICK_REGISTER_CONST_LONG("METRIC_PEAKABSOLUTEERROR", PeakAbsoluteErrorMetric);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("METRIC_PEAKSIGNALTONOISERATIO", PeakSignalToNoiseRatioMetric);
+#else
+	IMAGICK_REGISTER_CONST_LONG("METRIC_PEAKSIGNALTONOISERATIO", PeakSignalToNoiseRatioErrorMetric);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("METRIC_ROOTMEANSQUAREDERROR", RootMeanSquaredErrorMetric);
 #if MagickLibVersion >= 0x687
@@ -1301,18 +1395,32 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("METRIC_PERCEPTUALHASH_ERROR", PerceptualHashErrorMetric);
 #endif
 
+#if MagickLibVersion >= 0x707
+	IMAGICK_REGISTER_CONST_LONG("METRIC_STRUCTURAL_SIMILARITY_ERROR", StructuralSimilarityErrorMetric);
+	IMAGICK_REGISTER_CONST_LONG("METRIC_STRUCTURAL_DISSIMILARITY_ERROR", StructuralDissimilarityErrorMetric);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_CHAR", CharPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_CHAR", CharPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_DOUBLE", DoublePixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_DOUBLE", DoublePixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_FLOAT", FloatPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_FLOAT", FloatPixel);
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_INTEGER", IntegerPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_INTEGER", IntegerPixel);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_LONG", LongPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_LONG", LongPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_QUANTUM", QuantumPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_QUANTUM", QuantumPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_SHORT", ShortPixel);
+	IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_SHORT", ShortPixel);
 	//For now we deliberately DO NOT support the longlong pixel type.
 	//64 bit integers and PHP do not play nicely.
 	//IMAGICK_REGISTER_CONST_LONG("PIXEL_LONGLONG", ..);
+	//IMAGICK_REGISTER_CONST_LONG("PIXELSTORAGE_LONGLONG", ..);
+
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_UNDEFINED", UndefinedEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_ADD", AddEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_AND", AndEvaluateOperator);
@@ -1361,6 +1469,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_ROOT_MEAN_SQUARE", RootMeanSquareEvaluateOperator);
 #endif
 
+#if MagickLibVersion >= 0x70B
+    IMAGICK_REGISTER_CONST_LONG("EVALUATE_INVERSE_LOG", InverseLogEvaluateOperator);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_UNDEFINED", UndefinedColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_RGB", RGBColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_GRAY", GRAYColorspace);
@@ -1415,6 +1527,20 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 
 
+#if (MagickLibVersion >= 0x700 && MagickLibVersion >= 0x708) || (MagickLibVersion < 0x700 && MagickLibVersion >= 0x699)
+	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_LINEARGRAY", LinearGRAYColorspace);
+#endif
+
+#if MagickLibVersion >= 0x70B
+    IMAGICK_REGISTER_CONST_LONG("COLORSPACE_DISPLAYP3", DisplayP3Colorspace);
+    IMAGICK_REGISTER_CONST_LONG("COLORSPACE_ADOBE98", Adobe98Colorspace);
+    IMAGICK_REGISTER_CONST_LONG("COLORSPACE_PROPHOTO", ProPhotoColorspace);
+#endif
+
+#if MagickLibVersion >= 0x70A
+	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_JZAZBZ", JzazbzColorspace);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_UNDEFINED", UndefinedVirtualPixelMethod);
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_BACKGROUND", BackgroundVirtualPixelMethod);
 #if MagickLibVersion < 0x700
@@ -1439,6 +1565,13 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_VERTICALTILEEDGE", VerticalTileEdgeVirtualPixelMethod);
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_CHECKERTILE", CheckerTileVirtualPixelMethod);
 #endif
+
+	// These were missed, but have been present for 12 years, so assuming
+	// they are going to be available in all version of ImageMagick people
+	// are using.
+	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_DITHER", DitherVirtualPixelMethod);
+	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_RANDOM", RandomVirtualPixelMethod);
+
 	IMAGICK_REGISTER_CONST_LONG("PREVIEW_UNDEFINED", UndefinedPreview);
 	IMAGICK_REGISTER_CONST_LONG("PREVIEW_ROTATE", RotatePreview);
 	IMAGICK_REGISTER_CONST_LONG("PREVIEW_SHEAR", ShearPreview);
@@ -1517,6 +1650,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("RESOURCETYPE_HEIGHT", HeightResource);
 #endif
 
+#if (MagickLibVersion >= 0x700 && MagickLibVersion >= 0x708) || (MagickLibVersion < 0x700 && MagickLibVersion >= 0x69A)
+    IMAGICK_REGISTER_CONST_LONG("RESOURCETYPE_LISTLENGTH", ListLengthResource);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_UNRECOGNIZED", UnrecognizedDispose);
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_UNDEFINED", UndefinedDispose);
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_NONE", NoneDispose);
@@ -1552,6 +1689,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 #endif
 
+#if MagickLibVersion >= 0x701
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_NEAREST_PIXEL", NearestInterpolatePixel); /* NearestInterpolatePixel */
+#endif
+
 #if MagickLibVersion > 0x628
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_UNDEFINED", UndefinedLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_COALESCE", CoalesceLayer);
@@ -1605,6 +1746,11 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	// minor version.
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SENTINEL", SentinelDistortion);
 #endif
+
+#if MagickLibVersion >= 0x70B
+    IMAGICK_REGISTER_CONST_LONG("DISTORTION_RIGID_AFFINE", RigidAffineDistortion);
+#endif
+
 #if MagickLibVersion >= 0x644
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_BARRELINVERSE", BarrelInverseDistortion);
 #endif
@@ -1626,23 +1772,40 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 #if MagickLibVersion > 0x637
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_ACTIVATE", ActivateAlphaChannel);
-#if MagickLibVersion < 0x700
+#if MagickLibVersion >= 0x700
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_ON", OnAlphaChannel);
+#else
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_RESET", ResetAlphaChannel);
 #endif
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SET", SetAlphaChannel);
 #endif
 #if MagickLibVersion > 0x645
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_UNDEFINED", UndefinedAlphaChannel);
+#if MagickLibVersion >= 0x700
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DISCRETE", DiscreteAlphaChannel);
+#else
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_COPY", CopyAlphaChannel);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DEACTIVATE", DeactivateAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_EXTRACT", ExtractAlphaChannel);
+#if MagickLibVersion >= 0x700
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_OFF", OffAlphaChannel);
+#else
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_OPAQUE", OpaqueAlphaChannel);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SHAPE", ShapeAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_TRANSPARENT", TransparentAlphaChannel);
 #if MagickLibVersion >= 0x690
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_ASSOCIATE", AssociateAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DISSOCIATE", DisassociateAlphaChannel);
 #endif
+	/*
+	 DiscreteAlphaChannel,                  CopyAlphaChannel,
+	 DisassociateAlphaChannel,              DeactivateAlphaChannel,
+	 ExtractAlphaChannel,                   ExtractAlphaChannel,
+	 OffAlphaChannel,                       OpaqueAlphaChannel,
+	 OnAlphaChannel,                        ResetAlphaChannel,  // deprecated
+	*/
 
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_UNDEFINED", UndefinedColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_BARYCENTRIC", BarycentricColorInterpolate);
@@ -1671,7 +1834,7 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("FUNCTION_ARCSIN", ArcsinFunction);
 	IMAGICK_REGISTER_CONST_LONG("FUNCTION_ARCTAN", ArctanFunction);
 #endif
-#if MagickLibVersion >= 0x680
+#if MagickLibVersion >= 0x678
 #if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_FLATTEN", FlattenAlphaChannel);
 #endif
@@ -1687,9 +1850,13 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_MODE", ModeStatistic);
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_NONPEAK", NonpeakStatistic);
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_STANDARD_DEVIATION", StandardDeviationStatistic);
+#endif
 #if MagickLibVersion >= 0x690
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_ROOT_MEAN_SQUARE", RootMeanSquareStatistic);
 #endif
+
+#if MagickLibVersion >= 0x70C
+	IMAGICK_REGISTER_CONST_LONG("STATISTIC_CONTRAST", ContrastStatistic);
 #endif
 
 #ifdef IMAGICK_WITH_KERNEL
@@ -1788,6 +1955,17 @@ IMAGICK_REGISTER_CONST_LONG("NORMALIZE_KERNEL_CORRELATE", CorrelateNormalizeValu
 IMAGICK_REGISTER_CONST_LONG("NORMALIZE_KERNEL_PERCENT", PercentValue);
 #endif
 
+
+#if IM_HAVE_IMAGICK_GETSETIMAGEMASK
+IMAGICK_REGISTER_CONST_LONG("PIXELMASK_READ", ReadPixelMask);
+IMAGICK_REGISTER_CONST_LONG("PIXELMASK_WRITE", WritePixelMask);
+#endif
+
+
+#if MagickLibVersion >= 0x708
+IMAGICK_REGISTER_CONST_LONG("PIXELMASK_COMPOSITE", CompositePixelMask);
+#endif
+
 #undef IMAGICK_REGISTER_CONST_LONG
 #undef IMAGICK_REGISTER_CONST_STRING
 }
