@@ -3,7 +3,7 @@
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
    | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar               |
-   | Imagemagick (c) ImageMagick Studio LLC                               |
+   | ImageMagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -43,7 +43,7 @@ void php_imagick_pixel_iterator_new (PixelIterator *pixel_it, zval *return_value
 	php_imagickpixeliterator_object *internpix;
 
 	object_init_ex(return_value, php_imagickpixeliterator_sc_entry);
-	internpix = (php_imagickpixeliterator_object *) zend_object_store_get_object(return_value TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(return_value);
 
 	internpix->pixel_iterator = pixel_it;
 	internpix->initialized = 1;
@@ -56,7 +56,7 @@ void php_imagick_pixel_iterator_new (PixelIterator *pixel_it, zval *return_value
 /* {{{ proto ImagickPixelIterator ImagickPixelIterator::__construct(Imagick source)
    The ImagickPixelIterator constructor
 */
-PHP_METHOD(imagickpixeliterator, __construct)
+PHP_METHOD(ImagickPixelIterator, __construct)
 {
 	zval *magick_object;
 	php_imagickpixeliterator_object *internpix;
@@ -67,8 +67,8 @@ PHP_METHOD(imagickpixeliterator, __construct)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-	intern = (php_imagick_object *)zend_object_store_get_object(magick_object TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
+	intern = Z_IMAGICK_P(magick_object);
 
 	if (!intern->magick_wand) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "Invalid Imagick object passed" TSRMLS_CC);
@@ -97,7 +97,7 @@ PHP_METHOD(imagickpixeliterator, __construct)
 /* {{{ proto bool ImagickPixelIterator::resetIterator()
 	Resets the pixel iterator.  Use it in conjunction with PixelGetNextIteratorRow() to iterate over all the pixels in a pixel container.
 */
-PHP_METHOD(imagickpixeliterator, resetiterator)
+PHP_METHOD(ImagickPixelIterator, resetIterator)
 {
 	php_imagickpixeliterator_object *internpix;
 
@@ -105,7 +105,7 @@ PHP_METHOD(imagickpixeliterator, resetiterator)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -123,7 +123,7 @@ PHP_METHOD(imagickpixeliterator, resetiterator)
 /* {{{ proto bool ImagickPixelIterator::syncIterator()
 	Syncs the pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, synciterator)
+PHP_METHOD(ImagickPixelIterator, syncIterator)
 {
 	php_imagickpixeliterator_object *internpix;
 
@@ -131,7 +131,7 @@ PHP_METHOD(imagickpixeliterator, synciterator)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -146,7 +146,7 @@ PHP_METHOD(imagickpixeliterator, synciterator)
 /* {{{ proto bool ImagickPixelIterator::setIteratorFirstRow()
 	Sets the pixel iterator to the first pixel row.
 */
-PHP_METHOD(imagickpixeliterator, setiteratorfirstrow)
+PHP_METHOD(ImagickPixelIterator, setIteratorFirstRow)
 {
 	php_imagickpixeliterator_object *internpix;
 
@@ -154,7 +154,7 @@ PHP_METHOD(imagickpixeliterator, setiteratorfirstrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -172,7 +172,7 @@ PHP_METHOD(imagickpixeliterator, setiteratorfirstrow)
 /* {{{ proto bool ImagickPixelIterator::setIteratorLastRow()
 	Sets the pixel iterator to the last pixel row.
 */
-PHP_METHOD(imagickpixeliterator, setiteratorlastrow)
+PHP_METHOD(ImagickPixelIterator, setIteratorLastRow)
 {
 	php_imagickpixeliterator_object *internpix;
 
@@ -180,7 +180,7 @@ PHP_METHOD(imagickpixeliterator, setiteratorlastrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -196,9 +196,10 @@ PHP_METHOD(imagickpixeliterator, setiteratorlastrow)
 /* }}} */
 
 /* {{{ proto ImagickPixelIterator ImagickPixelIterator::getPixelIterator(Imagick source)
-	Returns a new pixel iterator, static method
+	Returns a new pixel iterator, static method. This is equivalent to Imagick::getPixelIterator
+	which probably ought to be used in preference to this method.
 */
-PHP_METHOD(imagickpixeliterator, getpixeliterator)
+PHP_METHOD(ImagickPixelIterator, getPixelIterator)
 {
 	PixelIterator *pixel_it;
 	zval *magick_object;
@@ -209,7 +210,7 @@ PHP_METHOD(imagickpixeliterator, getpixeliterator)
 		return;
 	}
 
-	intern = (php_imagick_object *)zend_object_store_get_object(magick_object TSRMLS_CC);
+	intern = Z_IMAGICK_P(magick_object);
 
 	if (!intern->magick_wand) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "Invalid Imagick object passed" TSRMLS_CC);
@@ -233,7 +234,7 @@ PHP_METHOD(imagickpixeliterator, getpixeliterator)
 /* {{{ proto ImagickPixelIterator ImagickPixelIterator::newPixelIterator(Imagick source)
 	Returns a new pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, newpixeliterator)
+PHP_METHOD(ImagickPixelIterator, newPixelIterator)
 {
 	PixelIterator *pixel_it;
 	zval *magick_object;
@@ -247,8 +248,8 @@ PHP_METHOD(imagickpixeliterator, newpixeliterator)
 
 	IMAGICK_METHOD_DEPRECATED_USE_INSTEAD("ImagickPixelIterator", "newPixelIterator", "ImagickPixelIterator", "getPixelIterator");
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-	intern = (php_imagick_object *)zend_object_store_get_object(magick_object TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis() );
+	intern = Z_IMAGICK_P(magick_object);
 
 	if (!intern->magick_wand) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "Invalid Imagick object passed" TSRMLS_CC);
@@ -279,9 +280,10 @@ PHP_METHOD(imagickpixeliterator, newpixeliterator)
 /* }}} */
 
 /* {{{ proto ImagickPixelIterator ImagickPixelIterator::getPixelRegionIterator(Imagick source, int x, int y, int columns, int rows)
-	Returns a new pixel region iterator, static method
+	Returns a new pixel region iterator, static method. This is equivalent to the non-static
+	Imagick::getPixelRegionIterator which ought to be used in preference to this method.
 */
-PHP_METHOD(imagickpixeliterator, getpixelregioniterator)
+PHP_METHOD(ImagickPixelIterator, getPixelRegionIterator)
 {
 	PixelIterator *pixel_it;
 	zval *magick_object;
@@ -293,7 +295,7 @@ PHP_METHOD(imagickpixeliterator, getpixelregioniterator)
 		return;
 	}
 
-	intern = (php_imagick_object *)zend_object_store_get_object(magick_object TSRMLS_CC);
+	intern = Z_IMAGICK_P(magick_object);
 
 	if (!intern->magick_wand) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "Invalid Imagick object passed" TSRMLS_CC);
@@ -317,7 +319,7 @@ PHP_METHOD(imagickpixeliterator, getpixelregioniterator)
 /* {{{ proto bool ImagickPixelIterator::newPixelRegionIterator(Imagick source, int x, int y, int columns, int rows)
 	Returns a new pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, newpixelregioniterator)
+PHP_METHOD(ImagickPixelIterator, newPixelRegionIterator)
 {
 	PixelIterator *pixel_it;
 	zval *magick_object;
@@ -332,8 +334,8 @@ PHP_METHOD(imagickpixeliterator, newpixelregioniterator)
 
 	IMAGICK_METHOD_DEPRECATED_USE_INSTEAD("ImagickPixelIterator", "newPixelRegionIterator", "ImagickPixelIterator", "getPixelRegionIterator");
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-	intern = (php_imagick_object *)zend_object_store_get_object(magick_object TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
+	intern = Z_IMAGICK_P(magick_object);
 
 	if (!intern->magick_wand) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "Invalid Imagick object passed" TSRMLS_CC);
@@ -366,14 +368,14 @@ PHP_METHOD(imagickpixeliterator, newpixelregioniterator)
 /* {{{ proto int ImagickPixelIterator::getIteratorRow()
 	Returns the crurent pixel iterator row.
 */
-PHP_METHOD(imagickpixeliterator, getiteratorrow)
+PHP_METHOD(ImagickPixelIterator, getIteratorRow)
 {
 	php_imagickpixeliterator_object *internpix;
 
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -392,7 +394,7 @@ PHP_METHOD(imagickpixeliterator, getiteratorrow)
 /* {{{ proto bool ImagickPixelIterator::setIteratorRow(int row)
 	Set the pixel iterator row.
 */
-PHP_METHOD(imagickpixeliterator, setiteratorrow)
+PHP_METHOD(ImagickPixelIterator, setIteratorRow)
 {
 	php_imagickpixeliterator_object *internpix;
 	MagickBooleanType status;
@@ -403,7 +405,7 @@ PHP_METHOD(imagickpixeliterator, setiteratorrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -427,20 +429,34 @@ static
 void s_pixelwands_to_zval (PixelWand **wand_array, unsigned long num_wands, zval *return_value TSRMLS_DC)
 {
 	php_imagickpixel_object *internp;
+#if PHP_VERSION_ID >= 70000
+	zval obj;
+#else
 	zval *obj;
+#endif
+
 	unsigned long i;
 
 	array_init(return_value);
 
 	for (i = 0; i < num_wands; i++) {
+#if PHP_VERSION_ID >= 70000
+		object_init_ex(&obj, php_imagickpixel_sc_entry);
+		internp = Z_IMAGICKPIXEL_P(&obj);
+#else 
 		MAKE_STD_ZVAL(obj);
 		object_init_ex(obj, php_imagickpixel_sc_entry);
+		internp = Z_IMAGICKPIXEL_P(obj);
+#endif
 
-		internp = (php_imagickpixel_object *) zend_object_store_get_object(obj TSRMLS_CC);
 		internp->initialized_via_iterator = 1;
 
 		php_imagick_replace_pixelwand(internp, wand_array[i]);
+#if PHP_VERSION_ID >= 70000
+		add_next_index_zval(return_value, &obj);
+#else
 		add_next_index_zval(return_value, obj);
+#endif
 	}
 
 }
@@ -448,7 +464,7 @@ void s_pixelwands_to_zval (PixelWand **wand_array, unsigned long num_wands, zval
 /* {{{ proto array ImagickPixelIterator::getPreviousIteratorRow()
 	Returns the previous row as an array of pixel wands from the pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, getpreviousiteratorrow)
+PHP_METHOD(ImagickPixelIterator, getPreviousIteratorRow)
 {
 	php_imagickpixeliterator_object *internpix;
 	PixelWand **wand_array;
@@ -458,7 +474,7 @@ PHP_METHOD(imagickpixeliterator, getpreviousiteratorrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -486,7 +502,7 @@ PHP_METHOD(imagickpixeliterator, getpreviousiteratorrow)
 /* {{{ proto array ImagickPixelIterator::getCurrentIteratorRow()
 	Returns the current row as an array of pixel wands from the pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, getcurrentiteratorrow)
+PHP_METHOD(ImagickPixelIterator, getCurrentIteratorRow)
 {
 	php_imagickpixeliterator_object *internpix;
 	PixelWand **wand_array;
@@ -496,7 +512,7 @@ PHP_METHOD(imagickpixeliterator, getcurrentiteratorrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -527,7 +543,7 @@ PHP_METHOD(imagickpixeliterator, getcurrentiteratorrow)
 /* {{{ proto array ImagickPixelIterator::getNextIteratorRow()
 	Returns the next row as an array of pixel wands from the pixel iterator.
 */
-PHP_METHOD(imagickpixeliterator, getnextiteratorrow)
+PHP_METHOD(ImagickPixelIterator, getNextIteratorRow)
 {
 	php_imagickpixeliterator_object *internpix;
 	PixelWand **wand_array;
@@ -537,7 +553,7 @@ PHP_METHOD(imagickpixeliterator, getnextiteratorrow)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -562,14 +578,14 @@ PHP_METHOD(imagickpixeliterator, getnextiteratorrow)
 /* {{{ proto bool ImagickPixelIterator::clear()
 	Clear resources associated with a PixelIterator.
 */
-PHP_METHOD(imagickpixeliterator, clear)
+PHP_METHOD(ImagickPixelIterator, clear)
 {
 	php_imagickpixeliterator_object *internpix;
 
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
@@ -588,7 +604,7 @@ PHP_METHOD(imagickpixeliterator, clear)
 /* {{{ proto bool ImagickPixelIterator::valid()
 	The if the current iterator row is valid
 */
-PHP_METHOD(imagickpixeliterator, valid)
+PHP_METHOD(ImagickPixelIterator, valid)
 {
 	php_imagickpixeliterator_object *internpix;
 
@@ -596,7 +612,7 @@ PHP_METHOD(imagickpixeliterator, valid)
 		return;
 	}
 
-	internpix = (php_imagickpixeliterator_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	internpix = Z_IMAGICKPIXELITERATOR_P(getThis());
 
 	if (!internpix->initialized) {
 		php_imagick_throw_exception (IMAGICKPIXELITERATOR_CLASS, "ImagickPixelIterator is not initialized correctly" TSRMLS_CC);
